//
// $Id$
//

#include <iostream>
#include "Hausdorff.h"

using namespace std;

#define DEBUG(x)





Hausdorff::Hausdorff()
{
  maxmax = maxmin = 0;
  avgmax = avgmin = 0;

  distance_type = MAXMIN;  // regular Hausdorff
  
}  // constructor



double
Hausdorff::operator()(const PointSet2D& source, const PointSet2D& target)
{
  return hausdorff_distance(source, target);
  
}  // Hausdorff::operator()



double
Hausdorff::directed_hausdorff_distance(const PointSet2D& source, const PointSet2D& target)
{
  // cout << "Hausdorff::directed_hausdorff_distance" << endl;
  
  int nr_source_points = source.get_nr_points();
  int nr_target_points = target.get_nr_points();
  
  double summax = 0;
  double summin = 0;
  maxmin = -1e9;
  maxmax = -1e9;
  
  for(int i=0; i < nr_source_points; i++) {
    double smallest = 1e9;
    double largest = 0;

    double x1 = source[i * 2];
    double y1 = source[i * 2 + 1];
    
    for(int j=0; j < nr_target_points; j++) {
      double x2 = target[j * 2];
      double y2 = target[j * 2 + 1];

      double dx = x1 - x2;
      double dy = y1 - y2;
      double dist = dx * dx + dy * dy;
      DEBUG(cout << "dist between (" << x1 << ", " << y1 << ") and (" << x2 << ", " << y2 << "): " << dist << endl);
      
      if (dist < smallest) smallest = dist;
      if (dist > largest) largest = dist;
    }

    summin += smallest;
    summax += largest;
    
    if (smallest > maxmin) maxmin = smallest;  // so this is max min
    if (largest > maxmax) maxmax = largest;    // and this is max max
  }

  avgmax = summax / nr_source_points;
  avgmin = summin / nr_source_points;

  switch(distance_type) {
    case MAXMIN: return maxmin;  // Hausdorff distance
    case MAXMAX: return maxmax;
    case AVGMIN: return avgmin;
    case AVGMAX: return avgmax;
    default:
      cout << "Error: unknown distance type in Hausdorff:directed_hausdorff_distance" << endl;
  }

  return -1;
  
}  // Hausdorff::directed_hausdorff_distance



double
Hausdorff::hausdorff_distance(const PointSet2D& source, const PointSet2D& target)
{

  double hd0 = directed_hausdorff_distance(source, target);
  double hd1 = directed_hausdorff_distance(target, source);
  //  cout << "Hausdorff::hausdorff_distance, hd0 = " << hd0 << ", hd1 = " << hd1 << endl;
  if (hd0 > hd1) return hd0;
  else return hd1;
  
}  // Hausdorff::hausdorff_distance

