//
// $Id$
//

#include <vector>
#include <iostream>
#include <fstream>
#include "data_types/common.h"

//
// use the traits class that operates on 2D pointsets,
// has a 2D isometry as the Map function, and the (max avg min)
// Hausdorff distance as the Error function
//
#include "traits/evo_traits_2d_iso_avg_hausdorff.h"

//
// typedef to save typing
//
typedef evo_traits_2d_iso_avg_hausdorff evo_traits;

//
// use default parameters for the genetic optimization
//
#include "lib/evo_par_default_traits.h"

#include "lib/EvoFunc.h"

//
// declare the class we're going to use
//
template class EvoFunc<evo_traits_2d_iso_avg_hausdorff, evo_par_default_traits>;

//
// both source and target are of the type "PointSet2D"
//
evo_traits::SourceType source;
evo_traits::TargetType target;
evo_traits gt;
EvoFunc<evo_traits, evo_par_default_traits> evo_opt(source, target, gt);


using namespace std;







int main(int argc, char **argv)
{
  cout << endl;
  cout << "--------" << endl;
  cout << "example1" << endl;
  cout << "--------" << endl;
  cout << endl;

  //
  // uncomment this line to have no output during optimization
  //
  //  evo_par_default_traits::VERBOSITY = 0;

  //
  // initialize three random transformation parameters
  //
  vector<double> parameters;
  parameters.push_back(MyRandom::uniform(-25, 25));
  parameters.push_back(MyRandom::uniform(-25, 25));
  parameters.push_back(MyRandom::uniform(0, M_PI / 2));
  cout << "Original parameters:" << endl;
  cout << "tx: " << parameters[0] << endl;
  cout << "ty: " << parameters[1] << endl;
  cout << "angle: " << parameters[2] << endl;
  cout << endl;
  
  //
  // set source to be a transformed version of the target
  //
  source = target;
  evo_traits::MapType& map = gt.map_object();
  map(source, target, parameters);
  
  //
  // now run the optimization (try to recover the random parameters
  // we used just now)
  //
  evo_opt.run();

  cout << endl;
  cout << endl;
  
  Indiv *ind_p = evo_opt.get_best_indiv();
  
  cout << "Parameters found by optimization in " << evo_opt.get_nr_iterations() << " iterations:" << endl;
  cout << "tx: " << (*ind_p)[0] << endl;
  cout << "ty: " << (*ind_p)[1] << endl;
  cout << "angle: " << (*ind_p)[2] << endl;
  cout << endl;
  
}  // main

