//
// $Id: Indiv.cc,v 1.3 2004/11/29 14:37:10 min Exp min $
//

#include <iostream>
#include <math.h>
#include "Indiv.h"
#include "MyRandom.h"

using namespace std;




Indiv::Indiv(int nr_pars) :
  nr_parameters(nr_pars)
{
  // these two calls should not be necessary (?)
  pars.clear();
  vars.clear();
  
  for(int i=0; i < nr_parameters; i++) {
    pars.push_back(0);
    vars.push_back(0);
  }
  
  tau = 1.0 / sqrt(2 * sqrt((double) nr_pars));
  tau2 = 1.0 / sqrt(2 * (double) nr_pars);

  parent = 0;
  
}  // constructor



Indiv::~Indiv()
{

}  // destructor



void
Indiv::init(vector<double>& parameter_ranges)
{
  int nr_pars = (int) parameter_ranges.size() / 2;
  //  cout << "Indiv::init, nr_pars = " << nr_pars << endl;
  
  for(int i=0; i < nr_pars; i++) {
    double min = parameter_ranges[i * 2];
    double max = parameter_ranges[i * 2 + 1];
    
    pars[i] = MyRandom::uniform(min, max);
    double range = max - min;
    vars[i] = MyRandom::uniform(0, range / 2.0) / sqrt((double) nr_pars);

  }  // for
  
}  // Indiv::init



void
Indiv::mutate()
{
  double normal_sample = MyRandom::gaussian();

  for(int i=0; i < nr_parameters; i++) {
    // first adjust variance
    double local_sample = MyRandom::gaussian();
    vars[i] = vars[i] * exp(tau2 * normal_sample + tau * local_sample);
    // then adjust parameters
    pars[i] = pars[i] + vars[i] * local_sample;
  }  // for

}  // Indiv::mutate



void
Indiv::recombine(Indiv* parent1, Indiv* parent2)
{
  std::vector<double>& pars1 = parent1->get_pars();
  std::vector<double>& pars2 = parent2->get_pars();
  std::vector<double>& vars1 = parent1->get_vars();
  std::vector<double>& vars2 = parent2->get_vars();

  for(int i=0; i < nr_parameters; i++) {
    pars[i] = pars1[i];
    vars[i] = (vars1[i] + vars2[i]) / 2;
  }
  
}  // Indiv::recombine




ostream& operator<<(ostream& out, const Indiv& ind)
{
  out << "[";
  int nr_pars = (int) ind.pars.size();
  for(int i=0; i < nr_pars; i++) {
    out << ind.pars[i];
    if (i < (nr_pars - 1)) out << " ";
  }
  out << "]";

  return out;
  
}  // Indiv::operator<<


