//
// $Id: Indiv.h,v 1.4 2004/11/29 14:37:10 min Exp min $
//

#ifndef __Indiv_h
#define __Indiv_h

#include <vector>

using namespace std;

//
// individual representation
//
// transformation parameters
//   variances, number of variances nv is [1, n]
//   covariances, number is 0, n(n-1)/2 if # variances is n,
//     or (n - nv/2)(nv - 1) for other values of nv
//
// to mutate, sample from normal distribution
//   variances and covariances are also adjusted (if present), from different distributions
//



class Indiv
{

public:

  Indiv(int nr_pars);
  ~Indiv();

  void init(vector<double>& parameter_ranges);

  int get_nr_pars() { return pars.size(); }
  std::vector<double>& get_pars() { return pars; }
  std::vector<double>& get_vars() { return vars; }

  double operator[] (int index) const { return pars[index]; }
  double& operator[] (int index) { return pars[index]; }

  int is_parent() { return parent; }
  void set_parent(int new_value) { parent = new_value; }
  
  void set_fitness(double new_value) { fitness = new_value; }
  double get_fitness() { return fitness; }
  
  void recombine(Indiv* parent1, Indiv* parent2);
  
  void mutate();
  

  friend ostream& operator<<(ostream& out, const Indiv& ind);

  
private:

  int nr_parameters;
  std::vector<double> pars;
  std::vector<double> vars;
  std::vector<double> covars;

  int parent;
  
  double fitness;
  double tau, tau2;
  
  
};  // Indiv class


#endif


