//
// $Id$
//

#include <iostream>
#include "data_types/Vector.h"
#include "data_types/Matrix.h"
#include "Map_2d_2d_Persp.h"

string Map_2d_2d_Persp::par_names[6] = {"translation x", "translation y", "d", "angle", "ux", "uy"};
double Map_2d_2d_Persp::initial_ranges[12] = {-25, 25, -25, 25, 0, 10, 0, 1.5708, -1, 1, -1, 1};

#define DEBUG(x)





Map_2d_2d_Persp::Map_2d_2d_Persp()
{
  Map::init_parameter_ranges(initial_ranges, 12);

}  // constructor



void
Map_2d_2d_Persp::operator()(const PointSet2D& source, PointSet2D& result)
{
  operator()(source, result, parameters);

}  // operator() without parameters



//
// parameters here are those of a 2D to 2D perspective transform (6)
//
// tx ty d angle ux uy
// u is a 3D unit vector, so uz can be derived from ux and uy
//
void
Map_2d_2d_Persp::operator()(const PointSet2D& source, PointSet2D& result,
			    const vector<double>& in_pars)
{
  parameters.clear();
  for(int i=0; i < 6; i++) parameters.push_back(in_pars[i]);
  
  double tx = parameters[0];
  double ty = parameters[1];
  double d = parameters[2];
  double angle = parameters[3];
  double ux = parameters[4];
  double uy = parameters[5];

  Vector u(ux, uy, 1.0);
  u.normalize();
  
  Matrix R;
  R.rotate(u, angle);

  Matrix T;
  T.translate(tx, ty, 0.0);

  Matrix M = R * T;
  M[11] = d;
  M[14] = 1.0;  // fix f
  M[15] = 1.0;
  
  //
  // transform source
  //
  int nr_points = source.get_nr_points();
  DEBUG(cout << "transforming " << nr_points << " points" << endl);
  result.clear();
  
  for(int i=0; i < nr_points; i++) {
    Vector P(source[i * 2], source[i * 2 + 1], 0, 1);
    DEBUG(cout << "source: " << P << endl);
    
    Vector Q = M * P;
    DEBUG(cout << "  after trans: " << Q << endl);
//     Q[X] /= Q[Z];
//     Q[Y] /= Q[Z];

    result.add_point(Q[X], Q[Y]);
    
  }  // for

}  // Map_2d_2d_Persp::operator()


