//
// $Id$
//

#include <iostream>
#include <math.h>
#include "data_types/Vector.h"
#include "data_types/Matrix.h"
#include "Map_2d_Iso.h"

string Map_2d_Iso::par_names[3] = {"translation x", "translation y", "angle"};
double Map_2d_Iso::initial_ranges[6] = {-25, 25, -25, 25, 0, 1.5708};

#define DEBUG(x)





Map_2d_Iso::Map_2d_Iso()
{
  Map::init_parameter_ranges(initial_ranges, 6);

}  // constructor



//
// parameters here are those of a 2D isometry
//
// tx ty angle
//
void
Map_2d_Iso::operator()(const PointSet2D& source, PointSet2D& result,
		       const vector<double>& in_pars)
{
  parameters.clear();
  for(int i=0; i < 3; i++) parameters.push_back(in_pars[i]);
  
  double tx = parameters[0];
  double ty = parameters[1];
  double angle = parameters[2];
  
  double cosa = cos(angle);
  double sina = sin(angle);
  
  Matrix M(cosa, -sina, 0.0, tx,
	   sina, cosa, 0.0, ty,
	   0.0, 0.0, 1.0, 0.0,
	   0.0, 0.0, 0.0, 1.0);

  //
  // transform source
  //
  int nr_points = source.get_nr_points();
  DEBUG(cout << "transforming " << nr_points << " points, pars: " << tx << ", " << ty << ", " << angle << endl);
  result.clear();
  
  for(int i=0; i < nr_points; i++) {
    Vector P(source[i * 2], source[i * 2 + 1]);
    
    Vector Q = M * P;
    DEBUG(if (i < 5) cout << "source: " << P << ",  after trans: " << Q << endl);

    result.add_point(Q[X], Q[Y]);
    
  }  // for

}  // Map_2d_Iso::operator()


