//
// $Id: Map_3d_Proj.cc,v 1.1 2004/03/01 09:46:54 min Exp min $
//

#include <iostream>
#include "Map_3d_Proj.h"

#define DEBUG(x) x





//
// parameters here are those of a 3D affine transformation
//
// tx ty tz sx sy sz rx ry rz
//
void
Map_3d_Proj::operator()(const PointSet3D& source, PointSet2D& result,
			const vector<double>& parameters)
{
  //
  // create affine transformation matrix
  //
  double rx = parameters[0];
  double ry = parameters[1];
  double rz = parameters[2];
  Matrix R;
  R.rotate(rx, ry, rz);
  
  double tx = parameters[3];
  double ty = parameters[4];
  double tz = parameters[5];
  Matrix T;
  T.translate(tx, ty, tz);
  
  double sx = parameters[6];
  double sy = parameters[7];
  double sz = parameters[8];
  Matrix S;
  S.scale(sx, sy, sz);
  
  Matrix M = T * S * R;
  DEBUG(cout << "3D affine transformation matrix: " << M << endl);
  
  //
  // transform source
  //
  int nr_points = source.get_nr_points();
  DEBUG(cout << "transforming " << nr_points << " points" << endl);
  result.clear();
  
  for(int i=0; i < nr_points; i++) {
    Vector P(source[i * 3], source[i * 3 + 1], source[i * 3 + 2]);
    DEBUG(cout << "source: " << P << endl);
    
    Vector Q = M * P;
    DEBUG(cout << "  after trans: " << Q << endl);
    
    Q = modelview * Q;
    DEBUG(cout << "  after modelv: " << Q << endl);
    
    Q = projection * Q;
    DEBUG(cout << "  after proj: " << Q << endl);
    
    Q[X] = Q[X] / Q[3];
    Q[Y] = Q[Y] / Q[3];
    Q[Z] = Q[Z] / Q[3];
    Q[3] = 1;
    DEBUG(cout << "  after div: " << Q << endl);

    result.add_point(Q[X], Q[Y]);
    
  }  // for
  
}  // Map::operator()



void
Map_3d_Proj::set_camera(Vector pos, Vector lookat, Vector up, float viewplane_dist)
{
  this->pos = pos;
  this->lookat = lookat;
  this->up = up;
  DEBUG(cout << "Map_3d_Proj::set_camera(" << pos << ", " << lookat << ", " << up << ")" << endl);
  
  this->viewplane_dist = viewplane_dist;
  DEBUG(cout << "  viewplane_dist: " << viewplane_dist << endl);
  
  compute_modelview_matrix();
  compute_projection_matrix();
  
}  // Map_3d_Proj::set_camera



void
Map_3d_Proj::compute_modelview_matrix()
{
  DEBUG(cout << "Map_3d_Proj::compute_modelview_matrix" << endl);

  // compute U,V,N (as in Foley and van Dam)
  Vector n = lookat - pos;
  n.normalize();
  Vector v = up;
  v.normalize();
  Vector u = n ^ v;
  u.normalize();
  
  // move camera position to origin
  Matrix translation;
  translation.translate(-pos[X], -pos[Y], -pos[Z]);

  modelview.set(u, v, n);
  modelview = modelview * translation;
  DEBUG(cout << "  modelview matrix: " << modelview << endl);
  
}  // Map_3d_Proj::compute_modelview_matrix



void
Map_3d_Proj::compute_projection_matrix()
{
  DEBUG(cout << "Map_3d_Proj::compute_projection_matrix" << endl);

  projection.identity();
  projection[14] = 1.0 / viewplane_dist;
  projection[15] = 0.0;
  DEBUG(cout << "  projection matrix: " << projection << endl);
    
}  // Map_3d_Proj::compute_projection_matrix


