//
// $Id: evo_traits_2d_aff_avg_hausdorff.h,v 1.5 2004/03/01 09:47:00 min Exp min $
//

#ifndef __EVO_TRAITS_2D_AFF_AVG_HAUSDORFF_H
#define __EVO_TRAITS_2D_AFF_AVG_HAUSDORFF_H

#include "data_types/PointSet2D.h"
#include "map_functions/Map_2d_Aff.h"
#include "error_functions/Avg_Hausdorff.h"





/// \brief an example of an instance (model) of the required traits for GenOpt
///
/// This instance uses PointSet2D as the source and target type,
/// a 2D affine transformation (using the CGAL library) as the map function,
/// and the "average Hausdorff" (or rather, the average minimum Hausdorff)
/// distance as the error function.
///
class evo_traits_2d_aff_avg_hausdorff
{

public:

  typedef class PointSet2D SourceType;
  typedef class PointSet2D TransformedSourceType;
  typedef class PointSet2D TargetType;

  typedef class Map_2d_Aff MapType;
  typedef class Avg_Hausdorff ErrorType;


  Map_2d_Aff
  map_object() const
  {
    return map;
  }

  Avg_Hausdorff
  error_object() const
  {
    return error;
  }

  
private:

  Map_2d_Aff map;
  Avg_Hausdorff error;

  
};  // evo_traits_2d_aff_avg_hausdorff class


#endif
